#include <stdio.h>
#include <algorithm>

/* It appears that fread() from the MSVC2015 runtime library
 * drops bytes when reading data piped to stdin.
 *
 * Steps to reproduce in command prompt:
 * - Compile the example:
 *   cl -o freadtest.exe main.cpp
 *
 * - Run on pipe fed from test.txt, writing to output file
 *   type test.txt | freadtest.exe out.txt
 *
 * - Compare test.txt/out.txt:
 * dir
 *     153,868 out.txt
 *     153,890 test.txt
 *
 * Standard error:
 *
 * _MSC_VER=1800
 * w.txt total=148890, newlines=5000
 *
 * _MSC_VER=1900
 * w.txt total=148881, newlines=4991
 */

int main(int argc, char *argv[])
{
#if defined(_MSC_VER)
    fprintf(stderr, "_MSC_VER=%d\n", _MSC_VER);
#endif

    if (argc < 2)
        return -1;
    FILE *f  = fopen(argv[1], "w");
    if (!f)
        return -2;
    char buf[16384];
    int total = 0;
    int newLineCount = 0;
    while (size_t r = fread(buf, 1, 16384, stdin)) {
        total += int(r);
        newLineCount += int(std::count(buf, buf + r, '\n'));
        fwrite(buf, 1, r, f);
    }
    fclose(f);
    fprintf(stderr, "%s total=%d, newlines=%d\n", argv[1], total, newLineCount);
    return 0;
}
